#include <avr\io.h>
#include <avr\interrupt.h>
#include <avr\sleep.h>
#include "lib_basic.h"
#include "lib_led_display.h"

#define TRIAC_ON	BIT_SET(PORT(D),5)
#define TRIAC_OFF	BIT_CLR(PORT(D),5)


#define TIMER_ON	TCNT0 = 0x00; TIMSK |= 0x02
#define TIMER_OFF	TIMSK &= ~0x02; TCNT0 = 0x00

#define INT0_LOW	MCUCR &= ~0x03
#define INT0_EDGE	MCUCR |= 0x03

#define IS_BUTTON_PRESS		(PIN(D) & 0x04) == 0x00
#define IS_SUPPLY_ON		(PIN(D) & 0x01) == 0x00
#define IS_SUPPLY_OFF		(PIN(D) & 0x01) != 0x00

#define AWAKE		0
#define GO_SLEEP	1

volatile unsigned char disp_number = 1;
volatile unsigned int time_value = 0;
volatile unsigned int licznik1 = 0;
volatile unsigned int licznik2 = 0;
volatile unsigned int licznik3 = 0;
volatile unsigned char stan = AWAKE;
volatile unsigned char temp = 0;


int main(void)
{
	//inicjalizacja wywietlacza LED
	Displey_Init();

	//wejscie przycisku
	PIN_INPUT(D,2);
	BIT_CLR(PORT(D),2);


	//wyjscie sterujace triak
	PIN_OUTPUT(D,5);
	TRIAC_OFF;

	//wejscie informujace o zasilaniu
	PIN_INPUT(D,0);
	BIT_CLR(PORT(D),0);

	//ustawienie licznika Timer0
	TCCR0A = 0x00;
	TCCR0B = 0x02;
	TIMER_OFF;

	//ustawienie przerwania INT0
	INT0_LOW;
	GIMSK |= 0x40;	
	
	//wlaczenie obslugi przerwan
	sei();

	//ustawienie dania przejcia do trybu upienia
	stan = GO_SLEEP;

	while(1)
	{
		//warunkowe wywietlanie informacji o czasie na wyswietaczu segmentowym
		if(
			(time_value > 0) &&
			(IS_SUPPLY_ON) &&
			(stan != GO_SLEEP)
		)
		{
			if(disp_number == 1)
				{Diplay_LED(time_value % 100 / 10, 1);}
			else
				{Diplay_LED(time_value % 10, 2);}
		}
		//naprzemienne wywietlanie dwch cyfr na wywietlaczach LED
		else if(
			(time_value == 0) &&
			(IS_SUPPLY_ON) &&
			(stan != GO_SLEEP) &&
			(licznik2 < 350)
		)
		{
			if(disp_number == 1)
			{
				Diplay_LED(11, 1);
				disp_number = 0;
			}
			else
			{
				Diplay_LED(11, 2);
				disp_number = 1;
			}
		}
		//naprzemienne wywietlanie dwch cyfr na wywietlaczach LED
		else if(
			(time_value == 0) &&
			(IS_SUPPLY_ON) &&
			(stan != GO_SLEEP)
		)
		{
			if(disp_number == 1)
			{
				Diplay_LED(10, 1);
				disp_number = 0;
			}
			else
			{
				Diplay_LED(10, 2);
				disp_number = 1;
			}
		}
		//zgaszenie wyswietlacza jesli urzdzenie jest zasilane z baterii
		else
		{
			Diplay_LED_Off();
		}

		//uspienie mikrokontrolera
		if(stan == GO_SLEEP)
		{
			TIMER_OFF;
			INT0_LOW;
			licznik1 = 0;
			licznik2 = 0;
			licznik3 = 0;
			time_value = 0;
			Diplay_LED_Off();
			TRIAC_OFF;
			set_sleep_mode(SLEEP_MODE_PWR_DOWN);
			sleep_mode();
		}
	}

	return 0;
}

ISR(TIMER0_OVF_vect,ISR_BLOCK)
{
	//odliczanie do 1 minuty
	//aktualnie odlicza jest 1 minuta
	licznik1++;
	if(licznik1 >= (30000)) //500 -> 1 sekunda
	{
		//zerowanie licznika
		licznik1 = 0;

		//warunkowa dekrementacja czasu
		if(time_value > 0)
		{
			time_value = time_value - 1;
			if(time_value == 0)
			{
				TIMER_OFF;
				stan = GO_SLEEP;
			}
		}
	}

	//kod przelaczajacy aktualnie wywietlan cyfr
	if(time_value > 0)
	{
		if(disp_number == 1)
			{disp_number = 2;}
		else
			{disp_number = 1;}
	}

	//odliczanie do wylaczenia licznika i wyzerowania go
	if(IS_BUTTON_PRESS)
	{
		if(licznik2 < 400)
			{licznik2++;}
		else
			{time_value = 0;}
	}

	//wymaga opoznienia w sprawdzeniu !!!!!
	if(licznik3 < 100)
	{
		licznik3++;
	}
	else if(IS_SUPPLY_OFF)
	{
		TIMER_OFF;
		time_value = 0;
		licznik1 = 0;
		licznik2 = 0;
		licznik3 = 0;
		Diplay_LED_Off();
		TRIAC_OFF;
		//odczekanie az zostanie puszczony przycisk
		while(IS_BUTTON_PRESS){};
		stan = GO_SLEEP;
	}

}

ISR(INT0_vect,ISR_BLOCK)
{
	//wybudzenie z trybu upienia
	if((MCUCR & 0x03) == 0x00)
	{
		stan = AWAKE;
		TRIAC_ON;
		INT0_EDGE;
		licznik1 = 0;
		licznik2 = 0;
		licznik3 = 0;
		TIMER_ON;
	}
	//zwikszenie czasu do wyczenia zasilania
	else if(licznik2 < 350)
	{
		//inkrementacja czasu o zadana wartosc
		//z ograniczeniem do maksymalnej - 90
		time_value += 10;
		if(time_value > 90)
			{time_value = 90;}
		licznik1 = 0;
		licznik2 = 0;
	}
	//wyczenie zasilania
	else
	{
		TIMER_OFF;
		licznik1 = 0;
		licznik2 = 0;
		licznik3 = 0;
		time_value = 0;
		stan = GO_SLEEP;
	}
}
